/**
    @file

    @brief
	WS2812 Example Application

	Control the two WS2812 Serially Addressable LEDs on the MM900 board.
**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2015, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */

/* INCLUDES ************************************************************************/

#include <ft900.h>
#include "ws2812.h"
#include "rgbhsv.h"

/* CONSTANTS ***********************************************************************/

#define N_LEDS (2)
#define VALUE (92)
//#define VALUE (255)
#define SATURATION (255)
#define STEP (24)

/* GLOBAL VARIABLES ****************************************************************/


/* LOCAL VARIABLES *****************************************************************/

hsv_t ledhsv[N_LEDS];
rgb_t ledrgb[N_LEDS];

/* MACROS **************************************************************************/

/* LOCAL FUNCTIONS / INLINES *******************************************************/

void setup(void);
void loop(void);

/* FUNCTIONS ***********************************************************************/

int main(void)
{
    setup();
    for(;;) loop();

    return 0;
}

void setup()
{
    uint8_t i;

    for (i=0; i<N_LEDS; ++i)
    {
        /* Every LED has a different Hue Offset */
        ledhsv[i].hue = (255/N_LEDS)*i;
        ledhsv[i].val = VALUE;
        ledhsv[i].sat = SATURATION;
    }

    ws2812_init();
}

void loop()
{
    uint8_t i;

    /* Fade to a new colour */
    for (i=0; i<N_LEDS; ++i)
    {
        hsv2rgb(ledhsv[i].hue, ledhsv[i].sat, ledhsv[i].val,
                &(ledrgb[i].r), &(ledrgb[i].g), &(ledrgb[i].b));
        ledhsv[i].hue++;
    }

    /* Write the new values to the LEDs */
    for (i=0; i<N_LEDS; ++i)
    {
        ws2812_write(ledrgb[i].r, ledrgb[i].g, ledrgb[i].b);
    }
    delayus(50); /* Have to wait this much for the WS2812 to latch their colour */

    delayms(25);
}

