/**
    @file

    @brief
	(Brief Text)

	(Description Text)
**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2015, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */

/* INCLUDES ************************************************************************/
#include "rgbhsv.h"

/* CONSTANTS ***********************************************************************/

/* GLOBAL VARIABLES ****************************************************************/

/* LOCAL VARIABLES *****************************************************************/

/* MACROS **************************************************************************/

/* LOCAL FUNCTIONS / INLINES *******************************************************/

/* FUNCTIONS ***********************************************************************/

void rgb2hsv(uint8_t r, uint8_t g, uint8_t b,
        uint8_t* h, uint8_t* s, uint8_t* v)
{
    /* Based on http://stackoverflow.com/a/14733008 */

    uint8_t rgbMin, rgbMax, rgbDiff;

    rgbMin = (r<g) ? ((r<b) ? r : b) : ((g<b) ? g : b);
    rgbMax = (r>g) ? ((r>b) ? r : b) : ((g>b) ? g : b);

    rgbDiff = rgbMax-rgbMin;

    *v = rgbMax;
    if (*v == 0)
    {
        *h = 0;
        *s = 0;
    }
    else
    {
        *s = 255 * rgbDiff / *v;
        if (*s == 0)
        {
            *h = 0;
        }
        else
        {
            if (rgbMax == r)
            {
                *h = 0 + 43 * (g-b)/(rgbDiff);
            }
            else if (rgbMax == g)
            {
                *h = 85 + 43 * (b-r)/(rgbDiff);
            }
            else
            {
                *h = 171 + 43 * (r-g)/(rgbDiff);
            }
        }
    }
}

void hsv2rgb(uint8_t h, uint8_t s, uint8_t v,
        uint8_t* r, uint8_t* g, uint8_t* b)
{
    /* Based on http://stackoverflow.com/a/14733008 */

    uint8_t region, remainder, p, q, t;

    if (s == 0)
    {
        *r = v;
        *g = v;
        *b = v;
    }
    else
    {
        region = h / 43;
        remainder = (h - (region * 43)) * 6;

        p = (v * (255 - s)) >> 8;
        q = (v * (255 - ((s * remainder) >> 8))) >> 8;
        t = (v * (255 - ((s * (255 - remainder)) >> 8))) >> 8;

        switch (region)
        {
        case 0:
            *r = v; *g = t; *b = p;
            break;
        case 1:
            *r = q; *g = v; *b = p;
            break;
        case 2:
            *r = p; *g = v; *b = t;
            break;
        case 3:
            *r = p; *g = q; *b = v;
            break;
        case 4:
            *r = t; *g = p; *b = v;
            break;
        default:
            *r = v; *g = p; *b = q;
            break;
        }
    }
}
